SystemOrganization addCategory: #'AL-Tests-Http'!

TestCase subclass: #ALClientTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2014 22:55'!
chunkedResponse
	^ 'HTTP/1.1 200 OK
Transfer-Encoding: chunked
Content-Type: application/json; charset=UTF-8

2
OK
0

' withInternetLineEndings asByteArray! !

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/22/2011 13:24'!
okResponse
	^ 'HTTP/1.1 200 OK
Content-Type: text/plain; charset=ASCII
Content-Length: 2

OK' withInternetLineEndings asByteArray! !

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 00:04'!
testAnswersResponse
	| client answer |
	client := ALClient host: 'localhost' port: 80 in: self okResponse reading out: nil writing.
	answer := client sendRequest: (ALRequest method: 'GET' url: ALUrl root).
	self assert: (answer isKindOf: ALResponse)! !

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 21:39'!
testCreationWithHostPort
	| client |
	client := ALClient host: 'localhost' port: 80.
	self assert: client host name = 'localhost'.
	self assert: client host port = 80.
	self deny: client isConnected.
	! !

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/24/2012 21:39'!
testCreationWithStreams
	| client |
	client := ALClient host: 'localhost' port: 80 in: [0] reading out: nil writing.
	self assert: client host name = 'localhost'.
	self assert: client host port = 80.
	self assert: client isConnected.
	! !

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/4/2012 09:58'!
testGet
	| client content |
	client := ALClient host: 'localhost' port: 80 in: self okResponse reading out: nil writing.
	content := client get: ALUrl root.
	self assert: content = 'OK'! !

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 00:02'!
testHost
	| client message out |
	out := ByteArray new writing.
	client := ALClient host: 'localhost' port: 80 in: self okResponse reading out: out.
	client sendRequest: (ALRequest get: ALUrl root).
	message := (out conclusion reading encoding: #latin1) rest.
	self assert: (message includesSubString: 'Host: localhost')! !

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 2/9/2014 21:17'!
testReadChunkedEntity
	| client content response |
	client := ALClient 
		host: 'localhost' 
		port: 80 
		in: self chunkedResponse reading 
		out: nil writing.
	response := client sendRequest: (ALRequest get: ALUrl root).
	response readEntityWith: [:in | content := in rest].
	self assert: content = 'OK'! !

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 00:02'!
testReadEntity
	| client content response |
	client := ALClient host: 'localhost' port: 80 in: self okResponse reading out: nil writing.
	response := client sendRequest: (ALRequest get: ALUrl root).
	response readEntityWith: [:in | content := in rest].
	self assert: content = 'OK'! !

!ALClientTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 00:04'!
testSendRequest
	| client out count |
	count := 0.
	out := [:byte | count := count + 1] writing.
	client := ALClient host: 'localhost' port: 80 in: self okResponse reading out: out.
	client sendRequest: (ALRequest method: 'GET' url: ALUrl root).
	self assert: count > 0! !

TestCase subclass: #ALConnectionTest
	instanceVariableNames: 'connection handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALConnectionTest methodsFor: 'emulating' stamp: 'cwp 12/2/2012 18:59'!
createRequestHandler
	^ self! !

!ALConnectionTest methodsFor: 'emulating' stamp: 'cwp 4/24/2011 21:59'!
handleRequest: aRequest
	^ handler value: aRequest! !

!ALConnectionTest methodsFor: 'support' stamp: 'cwp 3/20/2011 21:45'!
onRequest: aBlock
	| count |
	count := 0.
	handler := 
		[:request |
		count := count + 1.
		aBlock value: request].
	connection handleRequest.
	self assert: count > 0! !

!ALConnectionTest methodsFor: 'data' stamp: 'cwp 4/5/2011 20:24'!
request
	^ ByteArray new writing
		write: 'GET /griffle/plonk HTTP/1.1'; cr; lf;
		write: 'Host: localhost:8001'; cr; lf;
		write: 'Connection: keep-alive'; cr; lf;
		write: 'User-Agent: SUnit Test'; cr; lf;
		write: 'Accept: text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5'; cr; lf;
		write: 'Accept-Encoding: gzip,deflate,sdch'; cr; lf;
		write: 'Accept-Language: en-US,en;q=0.8'; cr; lf;
		write: 'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.3'; cr; lf;
		cr; lf;
		conclusion! !

!ALConnectionTest methodsFor: 'tests' stamp: 'cwp 10/8/2012 00:07'!
testParseChromeRequest

	connection := ALConnection 
		in: self request reading 
		out: ByteArray new writing
		application: self.
	self onRequest:
		[:request |
		self assert: request isMethodGet.
		self assert: request url = (ALUrl / 'griffle' / 'plonk').
		self assert: request isConnectionKeepAlive.
		self thisIsATest]! !

!ALConnectionTest methodsFor: 'support' stamp: 'cwp 4/24/2011 22:00'!
thisIsATest
	^ ALResponse status: 204 reason: 'This is a test'.! !

!ALConnectionTest methodsFor: 'emulating' stamp: 'cwp 11/30/2012 20:20'!
write:  aMessage into: aWriteStream
	aMessage writeInto: aWriteStream! !

TestCase subclass: #ALContentTypeParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALContentTypeParserTest methodsFor: 'as yet unclassified' stamp: 'cwp 7/11/2012 22:38'!
testTextHtml
	| in parser result |
	in := 'text/html' asByteArray reading.
	parser := ALHttpParser on: in.
	result := parser parse: ALContentType.
	self assert: (result isKindOf: ALParameterizedValue).
	self assert: result type = 'text'.
	self assert: result subtype = 'html'.
	self assert: result isText.
	self assert: result isHtml! !

!ALContentTypeParserTest methodsFor: 'as yet unclassified' stamp: 'cwp 7/11/2012 22:38'!
testTypeSubtype
	| in parser result |
	in := 'image/jpeg' asByteArray reading.
	parser := ALHttpParser on: in.
	result := parser parse: ALContentType.
	self assert: (result isKindOf: ALParameterizedValue).
	self assert: result type = 'image'.
	self assert: result subtype = 'jpeg'.
	self assert: result isImage.
	self assert: result isJpeg! !

!ALContentTypeParserTest methodsFor: 'as yet unclassified' stamp: 'cwp 7/11/2012 22:38'!
testTypeSubtypeParameter
	| in parser result |
	in := 'text/html; charset=ISO-8859-4' asByteArray reading.
	parser := ALHttpParser on: in.
	result := parser parse: ALContentType.
	self assert: (result isKindOf: ALParameterizedValue).
	self assert: result type = 'text'.
	self assert: result subtype = 'html'.
	self assert: result isText.
	self assert: result isHtml.
	self assert: result hasCharset.
	self assert: (result parameter: 'charset') = 'ISO-8859-4'.
	self assert: result charset = #'iso-8859-4'! !

TestCase subclass: #ALEtagTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALEtagTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/12/2013 19:29'!
testStrongEquality
	| one two |
	one := ALEtag strong: 'abc'.
	two := ALEtag strong: 'abc'.
	self assert: one = two! !

!ALEtagTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/12/2013 19:30'!
testStrongWeakInequality
	| one two |
	one := ALEtag strong: 'abc'.
	two := ALEtag weak: 'abc'.
	self deny: one = two! !

!ALEtagTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/12/2013 19:31'!
testTagInequality
	| one two |
	one := ALEtag strong: 'abc'.
	two := ALEtag strong: 'abd'.
	self deny: one = two! !

!ALEtagTest methodsFor: 'as yet unclassified' stamp: 'cwp 1/12/2013 19:30'!
testWeakEquality
	| one two |
	one := ALEtag weak: 'abc'.
	two := ALEtag weak: 'abc'.
	self assert: one = two! !

TestCase subclass: #ALFormDataParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALFormDataParserTest methodsFor: 'as yet unclassified' stamp: 'cwp 9/24/2011 14:42'!
testParseStream
	| result |
	result := ALFormDataParser parse: 'abc=def&ghi=jkl' reading.
	self assert: result isArray.
	self assert: result first key = 'abc'.
	self assert: result first value = 'def'.
	self assert: result second key = 'ghi'.
	self assert: result second value = 'jkl'! !

!ALFormDataParserTest methodsFor: 'as yet unclassified' stamp: 'cwp 9/24/2011 14:42'!
testParseString
	| result |
	result := ALFormDataParser parse: 'abc=def&ghi=jkl'.
	self assert: result isArray.
	self assert: result first key = 'abc'.
	self assert: result first value = 'def'.
	self assert: result second key = 'ghi'.
	self assert: result second value = 'jkl'! !

!ALFormDataParserTest methodsFor: 'as yet unclassified' stamp: 'cwp 9/24/2011 15:18'!
testUnexpectedReservedCharacter
	| result |
	result := ALFormDataParser parse: 'abc=%21def&ghi=jkl'.
	self assert: result first value = '!!def'! !

TestCase subclass: #ALHttpEntityTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALHttpEntityTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/30/2012 11:37'!
testPartialRead
	| entity stream |
	entity := ALHttpEntity reading: #[1 2 3].
	entity contentLength: 2.
	entity readWith: [:in | stream := in].
	self
		should: [stream get]
		raise: Incomplete! !

!ALHttpEntityTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/30/2012 11:37'!
testPartialWrite
	| entity stream |
	entity := ALHttpEntity callback:  [:out | stream := out].
	entity contentLength: 3.
	entity writeInto: [:ea | self assert: false] writing.
	self
		should: [stream put: 48]
		raise: Incomplete! !

!ALHttpEntityTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/30/2012 11:37'!
testReadLimit
	| entity content |
	entity := ALHttpEntity reading: #[1 2 3].
	entity contentLength: 2.
	entity readWith: [:in | content := in rest].
	self assert: content = #[1 2]! !

!ALHttpEntityTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/30/2012 11:37'!
testReadStackOrder
	"This test demonstrates that the read stream passed to the #readWith: block
	limits the stream to 3 bytes, then applies utf-8 decoding, rather than decoding
	first and limiting to stream to 3 characters. This matches the definition of
	Content-Length given by RFC2616."
	
	| entity content |
	entity := ALHttpEntity body: [#[195 161 98 99 100] reading].
	entity contentLength: 3.
	entity contentType: (ALContentType type: 'text' subtype: 'plain' charset: 'UTF-8') .
	content := entity readWith: [:in | in rest].
	self assert: content = 'áb'! !

!ALHttpEntityTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/30/2012 11:37'!
testReadUnlimited
	| entity content |
	entity := ALHttpEntity reading: #[1 2 3].
	entity readWith: [:in | content := in rest].
	self assert: content = #[1 2 3]! !

!ALHttpEntityTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/30/2012 11:37'!
testWriteLimit
	| entity count |
	entity := ALHttpEntity callback: 
		[:out | 
		out write: #[1 2].
		self should: [out put: 3] raise: Incomplete].
	entity contentLength: 2.
	count := 0.
	entity writeInto: [:byte | count := count + 1] writing.
	self assert: count = 2.! !

!ALHttpEntityTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/30/2012 11:37'!
testWriteStackOrder
	| entity stream bytes |
	stream := ByteArray new writing.
	(entity := ALHttpEntity callback:  [:out | out write: 'ábc'])
		contentLength: 4;
		contentType: (ALContentType type: 'text' subtype: 'plain' charset: 'UTF-8').
	entity writeInto: stream.
	bytes := stream conclusion.
	self assert: bytes = #[195 161 98 99]! !

TestCase subclass: #ALHttpMessageTest
	instanceVariableNames: 'message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALHttpMessageTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:38'!
assertAccessor: getter handles: anObject
	| setter value |
	setter := getter asMutator.
	message perform: setter with: anObject.
	value := message perform: getter.
	self assert: value = anObject! !

!ALHttpMessageTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:38'!
setUp
	message := ALResponse ok! !

!ALHttpMessageTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:41'!
testConnectionClose
	
	message beConnectionClose.
	self assert: message isConnectionClose! !

!ALHttpMessageTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:39'!
testConnectionKeepAlive
	
	message beConnectionKeepAlive.
	self assert: message isConnectionKeepAlive ! !

!ALHttpMessageTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:33'!
testContentLength
	self assertAccessor: #contentLength handles: 42! !

!ALHttpMessageTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:34'!
testContentType
	self assertAccessor: #contentType handles: ALContentType html! !

!ALHttpMessageTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 00:02'!
testHost
	message := ALRequest get: ALUrl root.
	self assertAccessor: #host handles: (ALHost name: 'localhost' port: 8000)! !

!ALHttpMessageTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/8/2012 00:02'!
testLocation
	self assertAccessor: #location handles: ALUrl root! !

!ALHttpMessageTest methodsFor: 'as yet unclassified' stamp: 'cwp 10/25/2011 13:41'!
testTransferEncodingChunked
	
	message beTransferEncodingChunked.
	self assert: message isTransferEncodingChunked.! !

TestCase subclass: #ALHttpParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALHttpParserTest methodsFor: 'data' stamp: 'cwp 12/13/2012 22:53'!
chromeAccept
	^ #[116 101 120 116 47 104 116 109 108 44 97 112 112 108 105 99 97 116 105 111 110 47 120 104 116 109 108 43 120 109 108 44 97 112 112 108 105 99 97 116 105 111 110 47 120 109 108 59 113 61 48 46 57 44 42 47 42 59 113 61 48 46 56]! !

!ALHttpParserTest methodsFor: 'data' stamp: 'cwp 10/24/2011 14:12'!
curlDirectoryRequest
	^ 'GET /plict/ HTTP/1.1
User-Agent: curl/7.19.7 (universal-apple-darwin10.0) libcurl/7.19.7 OpenSSL/0.9.8r zlib/1.2.3
Host: localhost:8000
Accept: */*

' withInternetLineEndings asByteArray ! !

!ALHttpParserTest methodsFor: 'data' stamp: 'cwp 10/24/2011 14:13'!
curlFileRequest
	^ 'GET /plict/ving HTTP/1.1
User-Agent: curl/7.19.7 (universal-apple-darwin10.0) libcurl/7.19.7 OpenSSL/0.9.8r zlib/1.2.3
Host: localhost:8000
Accept: */*

' withInternetLineEndings asByteArray! !

!ALHttpParserTest methodsFor: 'data' stamp: 'cwp 10/24/2011 14:13'!
curlRootRequest
	^ 'GET / HTTP/1.1
User-Agent: curl/7.19.7 (universal-apple-darwin10.0) libcurl/7.19.7 OpenSSL/0.9.8r zlib/1.2.3
Host: localhost:8000
Accept: */*

' withInternetLineEndings asByteArray! !

!ALHttpParserTest methodsFor: 'testing' stamp: 'cwp 7/12/2012 23:17'!
expectedFailures
	^ #(testFacebookSetCookies)! !

!ALHttpParserTest methodsFor: 'data' stamp: 'cwp 7/12/2012 16:01'!
facebookCookies
	^ 'c_user=6145454279; expires=Sat, 11-Aug-2012 22:59:41 GMT; path=/; domain=.facebook.com, datr=VjTLTDHJymBeI6yqW_8mhZ8x; expires=Sat, 12-Jul-2014 22:59:40 GMT; path=/; domain=.facebook.com; httponly, fr=0ENr3sp99YRTHFAWXegDkzOcZOpnf8_xsBjY6nE-M; expires=Sat, 11-Aug-2012 22:59:41 GMT; path=/; domain=.facebook.com; httponly, lsd=deleted; expires=Thu, 01-Jan-1970 00:00:01 GMT; path=/; domain=.facebook.com, lu=Tg6vl0PWAV5Whfh55oW1f1A; expires=Sat, 12-Jul-2014 22:59:41 GMT; path=/; domain=.facebook.com; httponly, reg_fb_gate=deleted; expires=Thu, 01-Jan-1970 00:00:01 GMT; path=/; domain=.facebook.com, reg_fb_ref=deleted; expires=Thu, 01-Jan-1970 00:00:01 GMT; path=/; domain=.facebook.com, s=Aa456fhh0q6BG4bha.BP_1bd; expires=Sat, 11-Aug-2012 22:59:41 GMT; path=/; domain=.facebook.com; secure; httponly, wd=deleted; expires=Thu, 01-Jan-1970 00:00:01 GMT; path=/; domain=.facebook.com; httponly, xs=1%3AFf7fghhTYzsNYIg%3A0%3A1342133981; expires=Sat, 11-Aug-2012 22:59:41 GMT; path=/; domain=.facebook.com; httponly' asByteArray! !

!ALHttpParserTest methodsFor: 'data' stamp: 'cwp 10/24/2011 14:13'!
googleResponse
	^ 'HTTP/1.1 200 OK
Date: Sat, 22 Oct 2011 19:07:01 GMT
Expires: -1
Cache-Control: private, max-age=0
Content-Type: text/html; charset=ISO-8859-1
Set-Cookie: PREF=ID=7ed57b7f2ef59632:FF=0:TM=1319310421:LM=1319310421:S=ZKyu_ijP4_xBlaEL; expires=Mon, 21-Oct-2013 19:07:01 GMT; path=/; domain=.google.ca
Set-Cookie: NID=52=DRXhH0J16bTsVAYzEQcRgL-O5vYAhYDNQ0fR1GztDI2MWikNPgMWqp0gl8iOLXS-aAPqFeACNbw5Agp-Z5jjzAVuBWEjxnRkBWgyJSPZF_LTFOgjEODCbdLAl5ZTRJbr; expires=Sun, 22-Apr-2012 19:07:01 GMT; path=/; domain=.google.ca; HttpOnly
Server: gws
X-XSS-Protection: 1; mode=block
X-Frame-Options: SAMEORIGIN
Transfer-Encoding: chunked

' withInternetLineEndings asByteArray! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 12/13/2012 22:56'!
testChromeAccept
	| in types |
	in := self chromeAccept reading.
	types := ALHttpParser parse: in as: ALContentTypes.
	self assert: types class = ALContentTypes.! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 10/25/2011 14:07'!
testCurlDirectoryRequest
	| in parser request |
	in := self curlDirectoryRequest reading.
	parser := ALHttpParser on: in.
	request := parser parse: ALRequest.
	self assert: request version = '1.1'.
	self assert: request isMethodGet.
	self assert: request host name = 'localhost'.
	self assert: request host port = 8000.
	! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 10/25/2011 14:10'!
testCurlFileRequest
	| in parser request |
	in := self curlFileRequest reading.
	parser := ALHttpParser on: in.
	request := parser parse: ALRequest.
	self assert: request version = '1.1'.
	self assert: request isMethodGet.
	self assert: request host name = 'localhost'.
	self assert: request host port = 8000.
	! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 10/25/2011 14:11'!
testCurlRootRequest
	| in parser request |
	in := self curlRootRequest reading.
	parser := ALHttpParser on: in.
	request := parser parse: ALRequest.
	self assert: request version = '1.1'.
	self assert: request isMethodGet.
	self assert: request host name = 'localhost'.
	self assert: request host port = 8000.
	! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 11/30/2012 19:44'!
testEtagStrong
	| in etag |
	in := '"abc"' asByteArray reading.
	etag := ALHttpParser parse: in as: ALEtag.
	self assert: (etag isKindOf: ALEtag).
	self assert: etag isStrong.
	self assert: etag tag = 'abc'! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 11/30/2012 20:19'!
testEtagWeak
	| in etag |
	in := 'W/"abc"' asByteArray reading.
	etag := ALHttpParser parse: in as: ALEtag.
	self assert: (etag isKindOf: ALEtag).
	self assert: etag isWeak.
	self assert: etag tag = 'abc'! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 7/12/2012 16:02'!
testFacebookSetCookies
	| in cookies |
	in := self facebookCookies reading.
	cookies := ALHttpParser parse: in as: ALCookieSpecs.
	self assert: (cookies isKindOf: Array).
	self assert: cookies size = 14.
	cookies do:
		[:c | 
		self assert: (c isKindOf: ALCookie).
		self assert: (c name isKindOf: String).
		self assert: (c value isKindOf: String).
		self deny: c parameters isEmpty]! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 10/24/2011 14:26'!
testGoogleResponse
	| in parser response |
	in := self googleResponse reading.
	parser := ALHttpParser on: in.
	response := parser parse: ALResponse.
	self assert: response version = '1.1'.
	self assert: response status = 200.
	self assert: response reason = 'OK'.
	self assert: response contentType = (ALContentType html charset: 'ISO-8859-1').! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 12/6/2012 16:47'!
testRfc1123DateTime
	"The parser answers in local time, so we have to convert the expected 
	date abd time to local time as well." 

	| actual expected in |
	in := 'Sun, 06 Nov 1994 08:49:37 GMT' asByteArray reading.
	expected := (DateAndTime year: 1994 month: 11 day: 6 hour: 8 minute: 49 second: 37)
					+ (Duration minutes: Locale current offsetLocalToUTC).
	actual := ALHttpParser parse: in as: DateAndTime.
	self assert: actual = expected! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 12/19/2012 23:07'!
testStrings
	| in strings |
	in := 'abc, def' asByteArray reading.
	strings := ALHttpParser parse: in as: ALStrings.
	self assert: (strings isKindOf: ALStrings).
	self assert: strings first = 'abc'.
	self assert: strings second = 'def'! !

!ALHttpParserTest methodsFor: 'tests' stamp: 'cwp 7/12/2012 15:33'!
testTwitterCookies
	| in cookies |
	in := self twitterCookies reading.
	cookies := ALHttpParser parse: in as: ALCookieValues.
	self assert: (cookies isKindOf: Array).
	self assert: cookies size = 14.
	cookies do:
		[:c | 
		self assert: (c isKindOf: ALCookie).
		self assert: (c name isKindOf: String).
		self assert: (c value isKindOf: String).
		self assert: c parameters isEmpty]! !

!ALHttpParserTest methodsFor: 'data' stamp: 'cwp 7/11/2012 21:34'!
twitterCookies
	^ 'guest_id=v1%3A133075865390530971; __qca=P0-731218367-1334985518443; pid=v3:13374052742446586882372; __utma=43835789.1201175464.1334985566.1337230683.1338568660.3; __utmz=43806368.1334985566.1.1.utmcsr=(direct)|utmccn=(direct)|utmcmd=(none); __utmv=43845368.lang%3A%20en; k=10.36.21.130.1341911493201780; auth_token_session=false; secure_session=true; twll=l%3D1341641509; remember_checked=1; js=1; lang=en; original_referer=padhuUp37zi4XoWoxyFqcGgJdw%2BJPXpx' asByteArray! !

TestCase subclass: #ALHttpPrinterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALHttpPrinterTest methodsFor: 'support' stamp: 'cwp 10/25/2011 14:18'!
assert: printBlock prints: aString
	| in printer |
	in := aString reading.
	printer := ALHttpPrinter on:
		[ :byte || expected actual |
		actual := byte asCharacter.
		expected := in get.
		self assert: actual = expected ] writing.
	printBlock value: printer.
	self assert: in available = 0! !

!ALHttpPrinterTest methodsFor: 'support' stamp: 'cwp 12/3/2012 20:13'!
gmt
	^ TimeZone offset:  0 hours name: 'Greenwich Mean Time' abbreviation: 'GMT'.! !

!ALHttpPrinterTest methodsFor: 'support' stamp: 'cwp 10/25/2011 14:14'!
rootRequest
	^ 'GET / HTTP/1.1<1s>User-Agent: Altitude/0.1<1s>Accept: */*<1s><1s>' expandMacrosWith: String crlf! !

!ALHttpPrinterTest methodsFor: 'tests' stamp: 'cwp 11/30/2012 19:40'!
testContentType
	
	self 
		assert: [:printer | ALContentType html printWith: printer]
		prints: ('text/html; charset=UTF-8') reading.! !

!ALHttpPrinterTest methodsFor: 'tests' stamp: 'cwp 11/30/2012 19:39'!
testCookie
	| cookies |
	cookies := ALCookieValues new: 2.
	cookies at: 1 put: (ALCookie name: 'abc' value: 'def').	
	cookies at: 2 put: (ALCookie name: 'ghi' value: 'jkl').
	self 
		assert: [:printer | cookies printWith: printer]
		prints: ('abc=def; ghi=jkl') reading.! !

!ALHttpPrinterTest methodsFor: 'tests' stamp: 'cwp 12/6/2012 10:19'!
testDateTime
	"This test (and the printing code it tests) has to work around Chronology's
	utterly broken timezone handling. DateAndTime instances are always created
	with an offset of 0, indicating the UTC timezone. Howerver, they are actually in
	in the local timezone, so to print a proper UTC time, the have to be converted first.

	We want the test (and the printing code) to work in any timezone, so we have to
	create a DateAndTime in such away that when it's converted to UTC we'll know
	what values it will have, and can predict how it will be printed."
	
	| dt |
	dt := DateAndTime fromString: '2012-12-03T11:33:10.205+00:00'.
	dt := dt + (Duration minutes: Locale current offsetLocalToUTC).
	self 
		assert: [:printer | dt printWith: printer]
		prints: 'Mon, 03 Dec 2012 11:33:10 GMT'! !

!ALHttpPrinterTest methodsFor: 'tests' stamp: 'cwp 11/30/2012 19:38'!
testETagStrong
	| etag |
	etag := ALEtag strong: 'abc'.
	self 
		assert: [:printer | etag printWith: printer]
		prints: ('"abc"') reading.! !

!ALHttpPrinterTest methodsFor: 'tests' stamp: 'cwp 11/30/2012 19:40'!
testETagWeak
	| etag |
	etag := ALEtag weak: 'abc'.
	self 
		assert: [:printer | etag printWith: printer]
		prints: ('W/"abc"') reading.! !

!ALHttpPrinterTest methodsFor: 'tests' stamp: 'cwp 11/30/2012 19:39'!
testPrintHeader
	
	self 
		assert: [:printer | printer printHeader: 'Content-Type' value: ALContentType html]
		prints: ('Content-Type: text/html; charset=UTF-8', String crlf) reading.! !

!ALHttpPrinterTest methodsFor: 'tests' stamp: 'cwp 10/8/2012 00:02'!
testRootRequest
	
	| request |
	request := ALRequest get: ALUrl root.
	self
		assert: [:printer | printer printRequest: request]
		prints: self rootRequest.
	! !

!ALHttpPrinterTest methodsFor: 'tests' stamp: 'cwp 11/30/2012 19:40'!
testSetCookie
	| cookies |
	cookies := ALCookieSpecs new: 2.
	cookies at: 1 put: ((ALCookie name: 'abc' value: 'def')
		domain: 'example.com';
		yourself).
	cookies at: 2 put: ((ALCookie name: 'ghi' value: 'jkl')
		domain: 'other.com';
		yourself).
	self 
		assert: [:printer | cookies printWith: printer]
		prints: ('abc=def; domain=example.com, ghi=jkl; domain=other.com') reading.! !

!ALHttpPrinterTest methodsFor: 'tests' stamp: 'cwp 12/19/2012 23:05'!
testStrings
	| strings |
	strings := ALStrings new: 2.
	strings at: 1 put: 'abc'.	
	strings at: 2 put: 'def'.
	self 
		assert: [:printer | strings printWith: printer]
		prints: 'abc, def'! !

TestCase subclass: #ALMessageTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

ALMessageTest subclass: #ALRequestTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALRequestTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2012 13:53'!
expect: aMimeMessage writtenAs: aByteArray
	| in |
	in := aByteArray reading.
	aMimeMessage writeInto:
		[:ea | | actual expected |
		expected := in get.
		actual := (ea isKindOf: Character)
			ifTrue: [ea asciiValue]
			ifFalse: [ea].
		self assert: actual = expected]
			writing.! !

!ALRequestTest methodsFor: 'as yet unclassified' stamp: 'cwp 12/30/2012 11:37'!
testChunking
	| message |
	message := ALRequest
		method: 'GET'
		url: ALUrl root
		version: 1.1s1
		body: (ALHttpEntity callback: [ :out | out write: 'abc' ]).
	message beTransferEncodingChunked.
	self 
		expect: message 
		writtenAs:  #[71 69 84 32 47 32 72 84 84 80 47 49 
					46 49 13 10 84 114 97 110 115 102 101 
					114 45 69 110 99 111 100 105 110 103 
					58 32 99 104 117 110 107 101 100 13 10 
					13 10 51 13 10 97 98 99 13 10 48 13 10
					13 10]
				! !

!ALRequestTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2012 13:33'!
testCreation
	| request |
	request := ALRequest method: 'GET' url: ALUrl root.
	self assert: request method = 'GET'.
	self assert: request url = ALUrl root.
	self assert: request version = 1.1s1! !

ALMessageTest subclass: #ALResponseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALResponseTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2012 13:40'!
expect: aMimeMessage writtenAs: expected
	| actual out |
	out := ByteArray new writing.
	aMimeMessage writeInto: out.
	actual := out conclusion.
	self assert: actual = expected! !

!ALResponseTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/3/2012 13:41'!
testCreation
	| response |
	response := ALResponse status: 204 reason: 'This is a test'.
	self assert: response status = 204.
	self assert: response reason = 'This is a test'.
	self assert: response version = 1.1s1! !

!ALResponseTest methodsFor: 'as yet unclassified' stamp: 'cwp 11/5/2012 00:06'!
testKeepAlive
	| response stream |
	response := ALResponse 
		type: ALContentType ascii 
		callback: [:out | out write: 'abc'].
	response beConnectionKeepAlive.
	stream := ByteArray new writing closing: 
		[self signalFailure: 'Shouldn''t close the stream'].
	response writeInto: stream.
	! !

TestCase subclass: #ALRelayTest
	instanceVariableNames: 'response handler writer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALRelayTest class methodsFor: 'as yet unclassified' stamp: 'cwp 9/8/2011 23:44'!
isAbstract
	^ self name = #ALRelayTest! !

!ALRelayTest methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 23:22'!
handleRequest: aRequest
	^ handler cull: aRequest! !

!ALRelayTest methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 23:22'!
setUp
	response := ALResponse status: 200 reason: 'OK'.
	handler := [response].
	writer := [:msg :ws | msg writeBodyInto: ws]! !

!ALRelayTest methodsFor: 'as yet unclassified' stamp: 'cwp 7/13/2012 23:23'!
write: aMessage into: aStream
	writer cull: aMessage cull: aStream! !

TestCase subclass: #ALUrlTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Tests-Http'!

!ALUrlTest methodsFor: 'asserting' stamp: 'cwp 10/7/2012 23:38'!
assertPath: aPath segments: anArray
	self assert: aPath class = ALPath.
	self assert: aPath size = anArray size.
	1 to: aPath size do: 
		[:index | self assert: (aPath at: index) = (anArray at: index)]! !

!ALUrlTest methodsFor: 'asserting' stamp: 'cwp 10/7/2012 23:36'!
assertUrl: aUrl tokens: anArray
	self assert: aUrl class = ALUrl.
	self assert: aUrl size = anArray size.
	1 to: aUrl size do:
		[:index |
		self assert: (aUrl at: index) = (anArray at: index)]! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 2/7/2014 19:35'!
testAddQueryBare
	| url base |
	base := ALUrl fromString: '/griffle'.
	url := base addQuery: 
		{'plonk' -> 'nurp'.
		'verny' -> 'wiffy'}.
	self assert: url printString = '/griffle?plonk=nurp&verny=wiffy'! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 2/7/2014 19:39'!
testAddQueryExisting
	| url base |
	base := ALUrl fromString: '/griffle?plonk=nurp'.
	url := base addQuery: {'verny' -> 'wiffy'}.
	self assert: url printString = '/griffle?plonk=nurp&verny=wiffy'! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 2/7/2014 19:39'!
testAddQueryExistingFragment
	| url base |
	base := ALUrl fromString: '/griffle?plonk=nurp#taff'.
	url := base addQuery: {'verny' -> 'wiffy'}.
	self assert: url printString = '/griffle?plonk=nurp&verny=wiffy#taff'! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 2/7/2014 19:38'!
testAddQueryFragment
	| url base |
	base := ALUrl fromString: '/griffle#taff'.
	url := base addQuery: 
		{'plonk' -> 'nurp'.
		'verny' -> 'wiffy'}.
	self assert: url printString = '/griffle?plonk=nurp&verny=wiffy#taff'! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/8/2012 00:09'!
testCreationWithPath
	| url |
	url := ALUrl withPath: (ALPath / 'griffle' / 'plonk').
	self assertUrl: url tokens: #($/ 'griffle' $/ 'plonk').! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:51'!
testEqual
	| one two |
	one := ALUrl fromString: 'http://localhost/griffle/plonk?a=b#wiffy'.
	two := ALUrl fromString: 'http://localhost/griffle/plonk?a=b#wiffy'.
	self assert: one = two! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:52'!
testHash
	| url |
	url := ALUrl fromString: 'http://localhost/griffle/plonk?a=b#wiffy'.
	self assert: url hash isInteger! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:36'!
testParsing
	| url |
	url := ALUrl fromString: '/griffle/plonk'.
	self assertUrl: url tokens: #($/ 'griffle' $/ 'plonk')! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:36'!
testParsingWithQuery
	| url |
	url := ALUrl fromString: '/griffle?nurp=oif'.
	self assertUrl: url tokens: #($/ 'griffle' $? 'nurp' $= 'oif')! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:37'!
testPath
	| url |
	url := ALUrl fromString: '/griffle/plonk'.
	self assertPath: url path segments: #('griffle' 'plonk').
! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:39'!
testPathWithFragment
	| url |
	url := ALUrl fromString: '/griffle/plonk#nurp'.
	self assertPath: url path segments: #('griffle' 'plonk')! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:40'!
testPathWithHost
	| url |
	url := ALUrl fromString: '//localhost/griffle/plonk'.
	self assertPath: url path segments: #('griffle' 'plonk')! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:39'!
testPathWithQuery
	| url |
	url := ALUrl fromString: '/griffle/plonk?q=2'.
	self assertPath: url path segments: #('griffle' 'plonk')! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:44'!
testPathWithScheme
	| url |
	url := ALUrl fromString: 'http:/griffle/plonk'.
	self assertPath: url path segments: #('griffle' 'plonk')! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:44'!
testPathWithSchemeHost
	| url |
	url := ALUrl fromString: 'http://localhost/griffle/plonk'.
	self assertPath: url path segments: #('griffle' 'plonk')! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/8/2012 14:16'!
testPrintEncoded
	| actual expected url |
	expected := 'http://localhost/grif%20fle'.
	url := ALUrl fromString: expected.
	actual := url printString.
	self assert: actual = expected.! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/8/2012 14:16'!
testPrintUnreserved
	| actual expected url |
	expected := 'http://localhost/griffle/plonk'.
	url := ALUrl fromString: expected.
	actual := url printString.
	self assert: actual = expected.! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/13/2012 23:51'!
testQuery
	| url query |
	url := ALUrl fromString: '/griffle?a=42&b=g'.
	query := url query.
	self assert: (query isKindOf: SequenceableCollection).
	self assert: (query size = 2).
	self assert: (query first value) = '42'.
	self assert: (query second value) = 'g'.
	! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 2/7/2014 19:33'!
testQueryWithFragment
	| url query |
	url := ALUrl fromString: '/griffle?a=42&b=g#quux'.
	query := url query.
	self assert: (query isKindOf: Array).
	self assert: (query size = 2).
	self assert: (query first value) = '42'.
	self assert: (query second value) = 'g'.
	! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:36'!
testRootConstruction
	| url |
	url := ALUrl root.
	self assertUrl: url tokens: #($/)! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:36'!
testSlashConstruction
	| url |
	url := ALUrl / 'griffle' / 'plonk'.
	self assertUrl: url tokens: #($/ 'griffle' $/ 'plonk')	! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:36'!
testSlashWithFragment
	| url base |
	base := ALUrl fromString: '/griffle#nurp'.
	url := base / 'plonk'.
	self assertUrl: url tokens: #($/ 'griffle' $/ 'plonk').! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:36'!
testSlashWithQuery
	| url base |
	base := ALUrl fromString: '/griffle?z=b'.
	url := base / 'plonk'.
	self assertUrl: url tokens: #($/ 'griffle' $/ 'plonk').! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:52'!
testUnequal
	| one two |
	one := ALUrl fromString: 'http://localhost/griffle/plonk?a=b#wiffy'.
	two := ALUrl fromString: 'http://localhost/griffle/plonk?a=c#wiffy'.
	self deny: one = two! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 10/7/2012 23:36'!
testWithConstruction
	| url |
	url := ALUrl with: 'gozer'.
	self assertUrl: url tokens: #('gozer')! !

!ALUrlTest methodsFor: 'tests' stamp: 'cwp 2/10/2014 11:32'!
testWriteIntoStream
	| actual elements url |
	elements := #('http' $: $/ $/ 'example.com' $/).	
	url := ALUrl new writing
		write: elements;
		conclusion.
	actual := Array new writing
		write: url;
		conclusion.
	self assert: elements = actual! !
